"use strict";
const websocket_1 = require("./websocket");
/**
 * Configuration schema https://developers.homebridge.io/#/config-schema
 */
/**
 * This example was derived from the Homebridge Accessory template.
 */
let hap;
/**
 * The maximum sauna temperature
 */
const MAX_TEMPERATURE = 120;
/**
 * the ID for the current temperature
 */
const CURRENT_TEMPERATURE_ID = 11;
/**
 * the ID for the target temperature
 */
const TARGET_TEMPERATURE_ID = 2;
/**
 * the ID for the firmware
 */
const FIRMWARE_ID = 21;
class SentiotecSaunaAccessory {
    /**
     * the constructor from the HAP API
     */
    constructor(log, config) {
        /**
         * the unique ID of the sauna
         */
        this.saunaId = 0;
        this.sentioAPI = new websocket_1.SentiotecAPI(log);
        this.log = log;
        this.config = config;
        this.saunaId = this.config.sauna;
        // temperature service
        this.temperaturService = new hap.Service.Thermostat(this.config.name);
        // current temperature
        this.temperaturService.getCharacteristic(hap.Characteristic.CurrentTemperature)
            .onGet(this.getCurrentTemperature.bind(this))
            .setProps({
            minValue: -20,
            maxValue: MAX_TEMPERATURE,
            minStep: 1,
        });
        // target temperature
        this.temperaturService.getCharacteristic(hap.Characteristic.TargetTemperature)
            .onGet(this.getTargetTemperature.bind(this))
            .on("set" /* SET */, (value, callback) => {
            log.info('Setting the sauna target temperature to ' + value);
            callback();
        })
            .setProps({
            minValue: 50,
            maxValue: MAX_TEMPERATURE,
            minStep: 1,
        });
        // temperature units
        this.temperaturService.getCharacteristic(hap.Characteristic.TemperatureDisplayUnits)
            .on("get" /* GET */, (callback) => {
            log.info('Getting sauna temperature units in Celsium (cannot be changed)');
            callback(undefined, hap.Characteristic.TemperatureDisplayUnits.CELSIUS);
        });
        // current cooling state - always on heating (no cooling sauna)
        this.temperaturService.getCharacteristic(hap.Characteristic.CurrentHeatingCoolingState)
            .on("get" /* GET */, (callback) => {
            callback(undefined, hap.Characteristic.CurrentHeatingCoolingState.HEAT);
        });
        this.temperaturService.getCharacteristic(hap.Characteristic.TargetHeatingCoolingState)
            .on("get" /* GET */, (callback) => {
            callback(undefined, hap.Characteristic.TargetHeatingCoolingState.HEAT);
        });
        this.temperaturService.setCharacteristic(hap.Characteristic.Name, this.config.name);
        this.informationService = new hap.Service.AccessoryInformation()
            .setCharacteristic(hap.Characteristic.Manufacturer, 'Sentiotec')
            .setCharacteristic(hap.Characteristic.Model, 'Pronet')
            .setCharacteristic(hap.Characteristic.Name, this.config.name)
            .setCharacteristic(hap.Characteristic.SerialNumber, this.config.serial)
            .setCharacteristic(hap.Characteristic.ProductData, 'Sauna heater with Pronet Web interface');
        this.informationService.getCharacteristic(hap.Characteristic.FirmwareRevision)
            .onGet(this.getFirmwareVersion.bind(this));
        log.info('Sauna finished initializing');
    }
    /**
     * This function retrieves a characteristic.
     * @param characteristicID the ID of the characteristic
     * @param characteristicName the human readable name of the characteristic for log output
     * @param converterFunction the converter function to get the correct value
     * @param characteristic the characteristic that should be udpated
     */
    /* eslint-disable @typescript-eslint/no-explicit-any*/
    getCharacteristic(characteristicID, characteristicName, converterFunction, characteristic) {
        if (this.sentioAPI.dataExpired) {
            // data has expired, so return the old value for now and then send an update to not provoke a timeout error
            this.sentioAPI.refreshCharacteristics(this.saunaId, characteristicID, this.config.ip, this.config.password, this.config.serial, this.log)
                .then(() => {
                if (!this.sentioAPI.connected) {
                    // Sauna is not connected to return an error
                    this.log.info('Sauna "' + this.saunaId + '" not connected');
                    characteristic.updateValue(new Error('Update characteristic failed: Sauna "' + this.saunaId + '" not connected'));
                }
                else {
                    const value = this.sentioAPI.getCachedCharacteristic(this.temperaturService, this.saunaId, characteristicID);
                    const convertedValue = converterFunction(value);
                    this.log.debug('Updating characteristic "' + characteristicName + '" with value :' + convertedValue);
                    characteristic.updateValue(convertedValue);
                }
            })
                .catch(error => characteristic.updateValue(error));
            return converterFunction(null);
        }
        else {
            // data is still current
            if (!this.sentioAPI.connected) {
                // Sauna is not connected to return an error
                this.log.info('Sauna "' + this.saunaId + '" not connected');
                return null;
            }
            else {
                const value = this.sentioAPI.getCachedCharacteristic(this.temperaturService, this.saunaId, characteristicID);
                const convertedValue = converterFunction(value);
                this.log.debug('Returning characteristic "' + characteristicName + '" with value :' + convertedValue);
                return convertedValue;
            }
        }
    }
    /**
     * This function returns the current temperature in the form of a callback.
     * @return the target temperature
     */
    getCurrentTemperature() {
        return this.getCharacteristic(CURRENT_TEMPERATURE_ID, 'Current Temperature', (value) => {
            if (value === null) {
                return 0;
            }
            else {
                return parseInt(value);
            }
        }, this.temperaturService.getCharacteristic(hap.Characteristic.CurrentTemperature));
    }
    /**
     * This function returns the target temperature in the form of a callback
     * @returns the target temperature
     */
    getTargetTemperature() {
        return this.getCharacteristic(TARGET_TEMPERATURE_ID, 'Target Temperature', (value) => {
            if (value === null) {
                return 0;
            }
            else {
                return parseInt(value);
            }
        }, this.temperaturService.getCharacteristic(hap.Characteristic.TargetTemperature));
    }
    /**
     * This function returns the software version of the Sauna control.
     * @returns the software version
     */
    getFirmwareVersion() {
        return this.getCharacteristic(FIRMWARE_ID, 'Firmware Version', (value) => {
            if (value === null) {
                return 'UNKNOWN';
            }
            else {
                return value;
            }
        }, this.informationService.getCharacteristic(hap.Characteristic.FirmwareRevision));
    }
    /*
     * This method is called directly after creation of this instance.
     * It should return all services which should be added to the accessory.
     */
    getServices() {
        return [
            this.informationService,
            this.temperaturService,
        ];
    }
}
module.exports = (api) => {
    hap = api.hap;
    api.registerAccessory('Sentiotec Sauna Control Plugin', SentiotecSaunaAccessory);
};
//# sourceMappingURL=accessory.js.map